#! /usr/bin/env python
import re
import sys

from scrib import barf


if sys.version_info[0] >= 3:
	xrange = range
	unicode = str
	basestring = str


class Clean:
	def __init__(self, debug):
		self.debug = debug
		self.chain_length = 2
	
	def sanitize(self, message):
		"""
		Sanitize the data. This is taken from Scrib 1.1.0 and could
		definitely use some tuning.
		"""
		if self.debug is 1:
			barf.Barf('DBG', 'Clean.sanitize: Sanitizing %s ' % message)
		things = [
			'://',
			'\"\'',
			"([a-zA-Z0-9\-_]+?\.)*[a-zA-Z0-9\-_]+?\.[a-zA-Z]{2,5}(\/[a-zA-Z0-9]*)*",
		]

		ret = message

		if self.debug is 1:
			barf.Barf('DBG', 'Clean.sanitize: Checking for newlines')
		for thing in things:
			ret = re.sub(thing, '', ret)
		
		ret = ret.replace("\n", " ")
		ret = ret.replace("\r", " ")

		if self.debug is 1:
			barf.Barf('DBG', 'Clean.sanitize: Scanning for color control codes...')
		# Strips out mIRC Control codes
		ccstrip = re.compile(
			"\x1f|\x02|\x12|\x0f|\x16|\x03(?:\d{1,2}(?:,\d{1,2})?)?",
			re.UNICODE
		)
		ret = ccstrip.sub("", ret)
		# Convert it all to ascii...
		ret = ret.encode('ascii', 'ignore')

		if self.debug is 1:
			barf.Barf('DBG', 'Clean.sanitize: Scanning for emoticons...')
		# Emotifacethings
		ret = ret.replace(": ", " : ")
		ret = ret.replace("; ", " ; ")
		# ^--- because some : and ; might be smileys...
		ret = ret.replace("`", "'")

		ret = ret.replace("?", " ? ")
		ret = ret.replace("!", " ! ")
		ret = ret.replace(".", " . ")
		ret = ret.replace(",", " , ")

		# Fixes broken emoticons...
		ret = ret.replace("^ . ^", "^.^")
		ret = ret.replace("- . -", "-.-")
		ret = ret.replace("O . o", "O.o")
		ret = ret.replace("o . O", "o.O")
		ret = ret.replace("o . o", "o.o")
		ret = ret.replace("O . O", "O.O")
		ret = ret.replace("< . <", "<.<")
		ret = ret.replace("> . >", ">.>")
		ret = ret.replace("> . <", ">.<")
		ret = ret.replace(": ?", ":?")
		ret = ret.replace(":- ?", ":-?")
		ret = ret.replace(", , l , ,", ",,l,,")
		ret = ret.replace("@ . @", "@.@")
		ret = ret.replace("D :", "D:")
		ret = ret.replace("c :", "c:")
		ret = ret.replace("C :", "C:")

		ret = ret.replace("||", "$C4")
		ret = ret.replace("|-:", "$b7")
		ret = ret.replace(":-|", "$b6")
		ret = ret.replace(";-|", "$b5")
		ret = ret.replace("|:", "$b4")
		ret = ret.replace(";|", "$b3")
		ret = ret.replace("=|", "$b2")
		ret = ret.replace(":|", "$b1")

		if ret is not None or ret is not 'None' or ret is not '':
			return self._unfilter(ret.strip())

	def _unfilter(self, message):
		"""
		Also taken from Scrib 1.1.0 and needs to be tuned.
		"""
		# Had to write my own initial capitalizing code *sigh*
		message = "%s%s" % (message[:1].upper(), message[1:])
		# Fixes punctuation
		message = message.replace(" ?", "?")
		message = message.replace(" !", "!")
		message = message.replace(" .", ".")
		message = message.replace(" ,", ",")
		message = message.replace(" : ", ": ")
		message = message.replace(" ; ", "; ")
		# Fixes I and I contractions
		message = message.replace(" i ", " I ")
		message = message.replace(" i'", " I'")

		message = message.replace("$C4", "||")
		message = message.replace("$b7", "|-:")
		message = message.replace("$b6", ";-|")
		message = message.replace("$b5", ":-|")
		message = message.replace("$b4", "|:")
		message = message.replace("$b3", ";|")
		message = message.replace("$b2", "=|")
		message = message.replace("$b1", ":|")

		# New emoticon filter that tries to catch almost all variations
		eyes, nose, mouth = r":;8BX=", r"-~'^O", r")(></\|CDPo39"  # NOQA
		# Removed nose from the pattern for the sake of my sanity
		pattern1 = "[\s][%s][%s][\s]" % tuple(map(re.escape, [eyes, mouth]))
		pattern2 = "[\s][%s][%s][\s]" % tuple(map(re.escape, [mouth, eyes]))
		eye, horzmouth = r"^><vou*@#sxz~-=+", r"-_o.wv"
		pattern3 = "[\s][%s][%s][%s][\s]" % tuple(map(
			re.escape,
			[eye, horzmouth, eye]
		))

		# Add whitespace for less false positives;
		# sit will be stripped out of the string later
		if not message == "":
			message = " " + message + " "
		emoticon = re.search(pattern1, message, re.IGNORECASE)
		pattern = pattern1
		if emoticon is None:
			emoticon = re.search(pattern2, message, re.IGNORECASE)
			pattern = pattern2
			if emoticon is None:
				emoticon = re.search(pattern3, message, re.IGNORECASE)
				pattern = pattern3
	
		# Init some strings so it does't barf later
		extra = ""
		emote = ""

		if emoticon is not None:
			emoticon = "%s" % emoticon.group()
			emotebeg = re.search(pattern, message, re.IGNORECASE).start()
			emoteend = re.search(pattern, message, re.IGNORECASE).end()
		
			# Remove the whitespace we added earlier
			message = message.strip()
		
			if not emotebeg == 0:
				emotebeg = emotebeg - 1
			if emotebeg == 0:
				emoteend = emoteend - 2
			emote = message[emotebeg:emoteend]
			if self.debug is 1:
				barf.Barf('DBG', "Clean._unfilter: Emote found: %s" % emote)

			new_message = message[:emotebeg]
			extra = message[emoteend:]
			message = new_message
		
			# Fixes the annoying XP capitalization in words...
			message = message.replace("XP", "xp")
			message = message.replace(" xp", " XP")
			message = message.replace("XX", "xx")
		
		if not message == "":
			# Remove whitespace if it wasn't removed earlier
			message = message.strip()
			if message.endswith(','):
				message = message[:-1]
			if not message.endswith(('.', '!', '?')):
				message = message + "."
			
		if not extra == "":
			extra = extra[1:]
			extra = "%s%s" % (extra[:1].upper(), extra[1:])
			if not extra.endswith(('.', '!', '?')):
				extra = extra + "."
			extra = extra + " "

		if not emote == "":
			message = message + extra + emote

		return message

	def split(self, message):
		"""
		Split message into word list!
		"""
		words = message.split()

		if len(words) > self.chain_length:
			for i in range(len(words) - self.chain_length):
				if self.debug is 1:
					barf.Barf('DBG', 'Clean.split: %s' % words[i:i + self.chain_length + 1])
				yield words[i:i + self.chain_length + 1]
