import re
import os
from setuptools import find_packages, setup


PKG = 'scrib'
VERSIONFILE = os.path.join(PKG, "version.py")
long_description = """Scrib is a chat bot library with the goal of becoming a
natural language assistant for everyday tasks.

Scrib is currently being ported to Python 3.
"""
install_requires = ['pymongo', 'beautifulsoup4']

verstr = "unknown"
try:
	verstrline = open(VERSIONFILE, "rt").read()
except EnvironmentError:
	pass # Okay, there is no version file.
else:
	VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
	mo = re.search(VSRE, verstrline, re.M)
	if mo:
		__version__ = mo.group(1)
	else:
		msg = "if %s.py exists, it is required to be well-formed" % VERSIONFILE
		raise RuntimeError(msg)

setup(
	name=PKG,
	version=__version__,
	install_requires=install_requires,
	entry_points={
		'console_scripts': [
			'scrib = scrib.gate:scrib_main'
		],
	},
	packages=find_packages(),
	package_dir={'scrib':'scrib'},
	package_data={
		'scrib.core': ['*.py'],
		'scrib.interfaces': ['*.py'],
	},
	author='Sina Mashek',
	author_email='mashek@thescoundrels.net',
	maintainer='Sina Mashek',
	maintainer_email='mashek@thescoundrels.net',
	long_description=long_description,
	description='Scrib is a chat bot framework.',
	license='MIT',
	url='http://scoundrels.github.io/scrib',
	platforms=['any'],
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Environment :: Console',
		'Environment :: Web Environment',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.6',
		'Programming Language :: Python :: 2.7',
	],
)
